/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a demo to convert images to PDF files.
 **/

#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main img2pdf
#endif

/* The default page size */
const float PageWidth	= 612.0;
const float PageHeight	= 792.0;
const float Margin		= 10.0;


// This function is called to insert an image into a page
FS_RESULT InsertImageObject(FSCRT_IMAGE image,FSCRT_PAGE page, FS_INT32 frameIndex)
{
	// Load an image based on frame index
	FS_RESULT ret = FSCRT_Image_LoadFrame(image, frameIndex);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSDK_OutputLog("Failure: In function FSCRT_Image_LoadFrame with error code %d.\r\n", ret);
		return ret;
	}

	// Create image object based on a page
	FSPDF_PAGEOBJECT imageObj = NULL;
	ret = FSPDF_ImageObject_Create(page, &imageObj);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSDK_OutputLog("Failure: In function FSPDF_ImageObject_Create with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return ret;
	}
	ret = FSPDF_ImageObject_SetImage(page, imageObj, image);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSPDF_PageObject_Release(page, imageObj);
		FSDK_OutputLog("Failure: In function FSPDF_ImageObject_SetImage with error code %d.\r\n", ret);
		return ret;
	}

	// Get the size of current image
	FS_INT32 width = 0;
	FS_INT32 height = 0; 
	ret = FSCRT_Image_GetCurrentFrameSize(image, &width, &height);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSPDF_PageObject_Release(page, imageObj);
		FSDK_OutputLog("Failure: In function FSCRT_Image_GetCurrentFrameSize with error code %d.\r\n", ret);
		return ret;
	}

	// the ratio of image width to page width
	float w = (float) width/PageWidth;
	// the ratio of image height to page height
	float h = (float) height/PageHeight;
	float scale = 1.0;
	float left = 0.0;
	float bottom = 0.0;

	// Calculate the position and scale of image object
	// which should be placed in a PDF
	if ( w > h ){		 
		if (w > 1.0){
			// Horizontal will be scaled to have the same width with page
			scale = 1/w;
			width = (FS_INT32)(width * scale - 2 * Margin);
			height = (FS_INT32)(height * scale);

			left = Margin;
			bottom = (PageHeight-height)/2;
		}
		else{
			// Both horizontal and vertical don't need to be scaled.
			left = (PageWidth-width)/2;
			bottom = (PageHeight-height)/2;
		}
	}
	else{
		// Vertical will be scaled to have the same height with page
		if( h> 1.0){
			scale = 1/h;
			width = (FS_INT32)(width * scale);
			height = (FS_INT32)(height * scale - 2 * Margin);
			left = (PageWidth-width)/2;
			bottom = Margin;
		}
		else{
			// Both horizontal and vertical don't need to be scaled.
			left = (PageWidth-width)/2;
			bottom = (PageHeight-height)/2;
		}
	}	

	// Compose an image matrix based on calculation as above
	FSCRT_MATRIX imageMatrix = {(FS_FLOAT)width, 0, 0, (FS_FLOAT)height, left, bottom};
	ret = FSPDF_PageObject_SetMatrix(page, imageObj, &imageMatrix);

	// Get page objects in a page and prepare to insert an image object
	FSPDF_PAGEOBJECTS pageObjs;
	ret = FSPDF_Page_GetPageObjects(page, &pageObjs);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSPDF_PageObject_Release(page, imageObj);
		FSDK_OutputLog("Failure: In function FSPDF_Page_GetPageObjects with error code %d.\r\n", ret);
		return ret;
	}

	// Insert an image object into page objects which in a page 
	ret = FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_ALL, 1, imageObj);
	if (ret != FSCRT_ERRCODE_SUCCESS){
		FSPDF_PageObject_Release(page, imageObj);
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_InsertObject with error code %d.\r\n", ret);
		return ret;
	}

	// Generate contents which are covered in page objects
	ret = FSPDF_PageObjects_GenerateContents(page, pageObjs);
	if (ret != FSCRT_ERRCODE_SUCCESS){
		FSPDF_PageObject_Release(page, imageObj);
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Image insert to PDF page.\r\n", ret);
	return ret;
}

void ImageToPDF(string fileName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Image with %s insert to PDF document ======\r\n", fileName.c_str());

	// Create a document at first
	CAutoDocument autoPdfDoc;
	FS_RESULT ret = autoPdfDoc.CreatePDFDocument();
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return;
	}

	FS_INT32 nFrameNum = 0;	
	string absFilePath = inputFilesPath + "/" + fileName;
	CAutoImage autoImage(absFilePath.c_str(), "rb");
	if ( NULL == autoImage.GetFile())
		return;

	// Get the image object using the file handler
	ret = autoImage.LoadImage();
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return;
	}
	
	// Load an image file to get the image object and frame number

	ret =FSCRT_Image_CountFrames(autoImage.GetImage(), &nFrameNum);
	if (FSCRT_ERRCODE_SUCCESS != ret){
		FSDK_OutputLog("Failure: In function FSCRT_Image_CountFrames with error code %d.\r\n", ret);
		return;
	}
	if(nFrameNum < 1){
		FSDK_OutputLog("Success: Image Count Frames: frameNum <= 0.\r\n");
		return;
	}
	FSDK_OutputLog("Success: Image have %d Frame(s).\r\n", nFrameNum);

	// For the image which has multiple frames,
	// The following cycle inserts each frame 
	// into different pages each time.
	for (FS_INT32 i = 0; i < nFrameNum; i++)
	{
		//load page
		CAutoPDFPage autoPDFPage;
		ret = autoPDFPage.CreatePDFPage(autoPdfDoc.GetDocument(),i);
		if (ret != FSCRT_ERRCODE_SUCCESS)
		{
			return;
		}
		// Set page size
		ret = FSPDF_Page_SetSize(autoPDFPage.GetPDFPage(), PageWidth, PageHeight);

		// Insert some frame of the image into page
		ret = InsertImageObject(autoImage.GetImage(), autoPDFPage.GetPDFPage(), i);

		if (ret != FSCRT_ERRCODE_SUCCESS){
			return;
		}
	}
	
	// Compose the output file name
 	string filename =  string(fileName.c_str(), fileName.length()-4) + ".pdf";
 	string outputFile = outputFilesPath + "/" + filename;
		
	// Save a document as a PDF file
	FSDK_SavePDFFile(autoPdfDoc.GetDocument(),outputFile.c_str(),1);
}

void  BrowseFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("Enter the input folder\r\n");

	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			// Check whether a image file is supported or not before calling ImageToPDF
			if(FSDK_IsSupportedImageFile(strFile.c_str(), TRUE)) 
				ImageToPDF(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;

	} while (TRUE);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	string inputSrcFile = "";
	string outputFolder = "";
	//Judge if to command line arguments. 
	//usage: img2pdf <srcfile> /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		if (argc >= 3)
		{
			if (0 != strcmp(argv[2], "/o"))
				bArgumentValid = false;
			else if (argc > 3)
				outputFolder = argv[3];
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: img2pdf [<srcfile> /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of an image file as input file.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/img2pdf";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("img2pdf");
		logPath = FSDK_GetLogFile("img2pdf");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: img2pdf\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(false);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string inputFolder;
	string imageName;
	if (inputSrcFile != "")
	{
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
		imageName = FSDK_GetFileName(inputSrcFile.c_str());
		ImageToPDF(imageName, inputFolder, outputFolder);
	}
	else
	{//Use default input path
		// Locate the folder where input files are
		inputFolder = FSDK_GetInputFilesFolder("img2pdf");
		// Open the folder
		void* folder = FSDK_OpenFolder(inputFolder.c_str(), FALSE);

		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/img2pdf: ");
			//Browse the given folder and deal with files one by one
			BrowseFolder(folder, inputFolder, outputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			imageName = "1.jpg";
			ImageToPDF(imageName, inputFolder, outputFolder);
 			imageName = "2.jpg";
 			ImageToPDF(imageName, inputFolder, outputFolder);
 			imageName = "TIF2Pages.tif";
 			ImageToPDF(imageName, inputFolder, outputFolder);
 			imageName = "3.jpg";
 			ImageToPDF(imageName, inputFolder, outputFolder);
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
