/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a demo to convert a PDF file to one or multiple image files .
 **/

#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdf2img
#endif

class CPDF2img
{
public:
	CPDF2img();
	~CPDF2img();

	FS_RESULT	AddFrameInTifFile(FSCRT_BITMAP	bitmap, FSCRT_IMAGEFILE imageFile);
	FS_RESULT	SaveImageFile(FSCRT_BITMAP bmp, string fileName, int index);
	void		BrowseFolder(void* folder);
	void		PDFToImage(string fileName);

public:
	string m_strPageRange;
	string m_strInputFile;
	string m_strInputFolder;
	string m_strOutputFolder;
	string m_strPdfName;
	FSCRT_FILE m_tifFile;
};

//"xxxxx_0001.xxxxx"
const int OUTPUT_FILE_NAME_EXT_LEN = 5; 
//The max page number is 9999
const int MAX_PAGE_INDEX = 9999;
const int MIN_PAGE_INDEX = 0;

/** @brief	Used for encode ARGB color format. */
#define FSCRT_ARGB_Encode(a,r,g,b)		((((FS_DWORD)(((FS_BYTE)(b) | ((FS_DWORD)((FS_BYTE)(g)) << 8)) | (((FS_DWORD)(FS_BYTE)(r)) << 16)))) | (((FS_DWORD)(FS_BYTE)(a)) << 24))

// image file type and name
typedef struct{
	FS_INT32	m_type;
	char*		m_extName;
}ImageFileType;

//Five image formats are supported to create in Foxit SDK. But only 4 of them are listed
//as below. It's because TIF supports multiple frames and needs to be handled in a special way.
ImageFileType imageFileType[] ={
								{FSCRT_IMAGETYPE_BMP,	(char*)".bmp"}, 
								{FSCRT_IMAGETYPE_JPG,	(char*)".jpg"},
								{FSCRT_IMAGETYPE_PNG,	(char*)".png"}, 
								{FSCRT_IMAGETYPE_JPX,	(char*)".jpx"},
								 };

CPDF2img::CPDF2img()
	:m_strPageRange("")
	,m_strInputFile("")
	,m_strInputFolder("")
	,m_strOutputFolder("")
	,m_strPdfName("")
{

};

CPDF2img::~CPDF2img()
{
	
};

//Add a frame in the TIF handler
FS_RESULT CPDF2img::AddFrameInTifFile (FSCRT_BITMAP	bitmap, FSCRT_IMAGEFILE imageFile)
{
	if(imageFile == NULL) 
	{
		FSDK_OutputLog("Failure: The parameter <imagefile> is invalid. Please check it.\n");
		return FSCRT_ERRCODE_PARAM;
	}
	return  FSCRT_ImageFile_AddFrame(imageFile, bitmap);
}

//Save a source bitmap data to the supported image files except the multiple frame TIF file.
FS_RESULT CPDF2img::SaveImageFile(FSCRT_BITMAP bmp, string fileName, FS_INT32 index)
{
	//Parameters checking
	if(bmp == NULL || index < MIN_PAGE_INDEX || index > MAX_PAGE_INDEX)
		return FSCRT_ERRCODE_PARAM;

	//If the length of file name is not larger than 4, return invalid file name
	if (fileName.length() <= 4){
		FSDK_OutputLog("Failure: The parameter <fileName> is invalid. Please check it.\r\n");
		return FSCRT_ERRCODE_PARAM;
	}

	//Generate image file name.
	string strFile = m_strOutputFolder;
	strFile += "/";
	strFile += string(fileName.c_str(), fileName.length()-4);
	char strExtName[OUTPUT_FILE_NAME_EXT_LEN+1] = {0};
	char buf[OUTPUT_FILE_NAME_EXT_LEN+1] = {0};
	strncpy(&strExtName[0], "_0000", OUTPUT_FILE_NAME_EXT_LEN);

	//The page index as one part of file name starts from 1.
	sprintf(buf, "%d", index+1);

	//Copy a string transformed from a frame index from buf to strExtName.
	FS_INT32 i = 0;
	for(char *p = buf; *p != '\0'; p++)
	{
		strExtName[OUTPUT_FILE_NAME_EXT_LEN-i-1]=*p;
		i++;
	}

	//Compose an absolute file name except the file extension. 
	strFile += string(strExtName, OUTPUT_FILE_NAME_EXT_LEN);

	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	//Save image files one by one
	for ( i = 0; i < sizeof(imageFileType)/sizeof(ImageFileType); i++)
	{
		string	imgFileName = strFile + imageFileType[i].m_extName;

		ret = FSDK_SaveImageFile(bmp, imageFileType[i].m_type, (const FS_LPSTR) imgFileName.c_str());
		//Result checking
		if (ret != FSCRT_ERRCODE_SUCCESS)
			return ret;
	}

	return ret;	
}

//It's a main control function to do transformation from a PDF file to image files
void CPDF2img::PDFToImage(string fileName)
{	
	FSDK_OutputLog("\r\n====== Convert %s to image file ======\r\n", fileName.c_str());
	// Compose the absolute input file path
	string strFile = m_strInputFolder + "/" + fileName;	

	// Mark the start time
	unsigned int nStartTime = FSDK_GetClock();

	CAutoDocument autoPdfDoc(strFile.c_str(), "rb");
	// Load PDF file
	if (FSCRT_ERRCODE_SUCCESS != autoPdfDoc.LoadPDFDocument(NULL))
		return;

	FS_INT32 nPageCount = 0;
	FS_INT32 nPageNum = 0;
	FS_INT32* nPageIndexArray = NULL;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if (m_strPageRange != "")
	{
		if (FSDK_GetPageRange(m_strPageRange.c_str(), nPageIndexArray, &nPageNum))
		{
			//Count page number
			ret = FSPDF_Doc_CountPages(autoPdfDoc.GetDocument(), &nPageCount);
			if (ret != FSCRT_ERRCODE_SUCCESS)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Doc_CountPages with error code %d.\r\n", ret);
				if (nPageIndexArray) 
					delete[] nPageIndexArray;
				nPageIndexArray = NULL;
				return;
			}
		}
	}
	if (nPageNum < 1) nPageNum = 1;

	//Parameters checking
	if (nPageNum < MIN_PAGE_INDEX || nPageNum > MAX_PAGE_INDEX) 
	{
		FSDK_OutputLog("Failure: The parameter <pageNum> is invalid. Please check it.\n");
		return;
	}
	//Wrong file name if the length of file name is not larger than 4
	if (fileName.length() <= 4){
		FSDK_OutputLog("Failure: The file name is invalid. Please check it.\r\n");
		return;
	}

	//Compose the absolute file path
	string strtifFile =  m_strOutputFolder + "/" + string(fileName.c_str(),fileName.length()-4) + ".tif";
	//Open the output file to prepare for writing
	CAutoDocument tifFile(strtifFile.c_str(), "wb+");

	CAutoImage autoImage;
	autoImage.CreateImageFile(tifFile.GetFileHander(), FSCRT_IMAGETYPE_TIF, nPageNum);
	
	//Dealing with page one by one
	FS_INT32 nFinishPageCount = 0;
	for (int i=0; i < nPageNum; i++)
	{
		CAutoPDFPage autoPdfPage;
		//Get a page handler based on the given document and index
		FS_INT32 pageIndex;

		if (m_strPageRange != "" && nPageIndexArray)
		{
			if (nPageIndexArray[i] >= nPageCount|| nPageIndexArray[i] < 0)
			{
				FSDK_OutputLog("Failure: Page index %d is out of valid range!\r\n", nPageIndexArray[i]);
				continue;
			}
			pageIndex = nPageIndexArray[i];
		}
		else
		{
			pageIndex = i;
		}
		ret  = autoPdfPage.LoadPDFPage(autoPdfDoc.GetDocument(),pageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}

		//Before loading a text page from a PDF page, the PDF page should be parsed at first.
		autoPdfPage.ParsePage();
		//Covert a page to a bitmap
		ret = autoImage.PageToBitmap(autoPdfPage.GetPDFPage());
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}

		//Save the given bitmap to the supported image file
		SaveImageFile(autoImage.GetBitmap(), fileName, pageIndex);
		//Add a page to a TIF frame
		AddFrameInTifFile(autoImage.GetBitmap(), autoImage.GetImageFile());
		nFinishPageCount++;
	}

	if (nPageIndexArray)
	{
		delete []nPageIndexArray;
		nPageIndexArray = NULL;
	}

	//Mark the end time
	unsigned int nEndTime = FSDK_GetClock();

	//Calculate the duration
	FSDK_OutputLog("Success: The %d page(s) convert to image file finished.\r\n\t Elapsed time: %dms.\r\n", nFinishPageCount, nEndTime - nStartTime);
}

void  CPDF2img::BrowseFolder(void* folder)
{
	FSDK_OutputLog("Enter the input folder\r\n");
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			// Check whether a file is PDF or not
			if(FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
				//PDFToImage(strFile, inputFilesPath, outputFilesPath);
				PDFToImage(strFile);
		}
		if (strFile.size() < 1) break;

	} while (TRUE);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	CPDF2img pdf2img;
	//Judge if to command line arguments. 
	//usage: pdf2img <srcfile> /r [pagerange] /o [destfolder]
	if (argc > 1)
	{
		pdf2img.m_strInputFile = argv[1];
		bool bArgumentValid = true;
		switch(argc)
		{
		case 2:
			break;
		case 3:
			{
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				break;
			}
		case 4:
			{//usage: pdf2img <srcfile> XXX XXX
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				else if (0 == strcmp(argv[2], "/r"))
				{
					if (0 != strcmp(argv[3], "/o") && '/'!=argv[3][0])
						pdf2img.m_strPageRange = argv[3]; //usage: pdf2img <srcfile> /r [pagerange]
					else if (0 != strcmp(argv[3], "/o") && '/'==argv[3][0])
						bArgumentValid = false;
				}
				else if (0 == strcmp(argv[2], "/o"))
					pdf2img.m_strOutputFolder = argv[3]; //usage: pdf2img <srcfile> /o [destfolder]
				break;
			}
		case 5:
			{//usage: pdf2img <srcfile> /r XXX XXX
				if (0 != strcmp(argv[2], "/r"))
					bArgumentValid = false;
				else 
				{
					if (0 != strcmp(argv[3], "/o"))
					{
						if (0 != strcmp(argv[4], "/o"))
							bArgumentValid = false;
						else
							pdf2img.m_strPageRange = argv[3]; //usage: pdf2img <srcfile> /r [pagerange] /o
					}
					else
						pdf2img.m_strOutputFolder = argv[4];//usage: pdf2img <srcfile> /r /o [destfolder]
				}
				break;
			}
		case 6:
			{//usage: pdf2img <srcfile> /r [pagerange] /o [destfolder]
				if (0 != strcmp(argv[2], "/r") || 0 != strcmp(argv[4], "/o"))
					bArgumentValid = false;
				else
				{
					pdf2img.m_strPageRange = argv[3];
					pdf2img.m_strOutputFolder = argv[5];
				}
				break;
			}
		default:
			bArgumentValid = false;
		}
		if (false == bArgumentValid || !FSDK_FileExist(pdf2img.m_strInputFile.c_str()) || (pdf2img.m_strOutputFolder != "" && !FSDK_FolderExist(pdf2img.m_strOutputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdf2img [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
				"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (pdf2img.m_strOutputFolder != "")
	{
		pdf2img.m_strOutputFolder += "/pdf2img";
		FSDK_CreatFolder(pdf2img.m_strOutputFolder.c_str());
		logPath = pdf2img.m_strOutputFolder;
		logPath += "/log.txt";
	}
	else
	{
		pdf2img.m_strOutputFolder = FSDK_GetOutputFilesFolder("pdf2img");
		logPath = FSDK_GetLogFile("pdf2img");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdf2img\r\n\r\n");

	//Initialize Foxit PDF SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(pdf2img.m_strOutputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	if (pdf2img.m_strInputFile != "")
	{
		pdf2img.m_strInputFolder = FSDK_GetCustomInputPath(pdf2img.m_strInputFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", pdf2img.m_strInputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdf2img.m_strOutputFolder.c_str());
		pdf2img.m_strPdfName = FSDK_GetFileName(pdf2img.m_strInputFile.c_str());
		pdf2img.PDFToImage(pdf2img.m_strPdfName);
	}
	else
	{
		//Locate the input file folder
		pdf2img.m_strInputFolder = FSDK_GetInputFilesFolder("pdf2img");
		//Open the input file folder
		void* folder = FSDK_OpenFolder(pdf2img.m_strInputFolder.c_str(), FALSE);
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", pdf2img.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdf2img.m_strOutputFolder.c_str());
			//Browse the given folder and deal with files one by one
			pdf2img.BrowseFolder(folder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdf2img.m_strInputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", pdf2img.m_strInputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdf2img.m_strOutputFolder.c_str());
			pdf2img.m_strPdfName = "Pages2TIF.pdf";
			pdf2img.PDFToImage(pdf2img.m_strPdfName);
			pdf2img.m_strPdfName = "PDF2Img.pdf";
			pdf2img.PDFToImage(pdf2img.m_strPdfName);
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();

	return 0;
}
